
/*
Seth Siemens
ME 708
Individual Project
This code has been adapted from the "piano" example in the NoodleSynth library
created by nathanRamaNoodles.
Modified 05-09-2024 by Seth Siemens
*/

/*
  MIT License

  Copyright (c) 2018 nathanRamaNoodles

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
//To learn more about this project go to https://github.com/nathanRamaNoodles/MusicWithoutDelay-LIbrary

#include <SensorToButton.h>  //https://github.com/nathanRamaNoodles/SensorToButton  acquired by JChristensen's popular button library
#include <NoodleSynth.h>

//function declarations
void buttonToPiano(SensorToButton b, MusicWithoutDelay &m);
int buttonToToneSelect(SensorToButton b, int Wave);

//Set synth engine
synthEngine mixer(20E3);

//Create variables to hold the notes that we want to play.
const char note1[] PROGMEM = {"::c"};   //plays c
const char note2[] PROGMEM = {"::c#"};  //plays c#
const char note3[] PROGMEM = {"::d"};   //plays d
const char note4[] PROGMEM = {"::d#"};  //plays d#... and so on
const char note5[] PROGMEM = {"::e"}; 
const char note6[] PROGMEM = {"::f"};  
const char note7[] PROGMEM = {"::f#"};  
const char note8[] PROGMEM = {"::g"};
const char note9[] PROGMEM = {"::g#"};
const char note10[] PROGMEM = {"::a"};
const char note11[] PROGMEM = {"::a#"};
const char note12[] PROGMEM = {"::b"};
const char note13[] PROGMEM = {"::c1"};
const char note14[] PROGMEM = {"::d1b"};
const char note15[] PROGMEM = {"::d1"};
//const char note16[] PROGMEM = {"::d1#"};
//const char note17[] PROGMEM = {"::e1"};

//initialize all the notes with the MusicWithoutDelay library
MusicWithoutDelay pianoKey(note1);
MusicWithoutDelay pianoKey2(note2);
MusicWithoutDelay pianoKey3(note3);
MusicWithoutDelay pianoKey4(note4);
MusicWithoutDelay pianoKey5(note5);
MusicWithoutDelay pianoKey6(note6);
MusicWithoutDelay pianoKey7(note7);
MusicWithoutDelay pianoKey8(note8);
MusicWithoutDelay pianoKey9(note9);
MusicWithoutDelay pianoKey10(note10);
MusicWithoutDelay pianoKey11(note11);
MusicWithoutDelay pianoKey12(note12);
MusicWithoutDelay pianoKey13(note13);
MusicWithoutDelay pianoKey14(note14);
MusicWithoutDelay pianoKey15(note15);
//MusicWithoutDelay pianoKey16(note16);
//MusicWithoutDelay pianoKey17(note17);

//Name button pins that are being used on the Arduino
#define BUTTON_PIN1 22    
#define BUTTON_PIN2 23       
#define BUTTON_PIN3 24
#define BUTTON_PIN4 25
#define BUTTON_PIN5 26
#define BUTTON_PIN6 27
#define BUTTON_PIN7 28
#define BUTTON_PIN8 29
#define BUTTON_PIN9 30
#define BUTTON_PIN10 31
#define BUTTON_PIN11 32
#define BUTTON_PIN12 33
#define BUTTON_PIN13 34
#define BUTTON_PIN14 35
#define BUTTON_PIN15 36
#define BUTTON_PIN16 37
#define BUTTON_PIN17 38
#define SELECTOR_PIN 2

#define DEBOUNCE_MS 70     //A debounce time of 70 milliseconds usually works well for noisy button switches. if not, try 20.

//Delcare the buttons with SensorToButton library
SensorToButton toneSelect(SELECTOR_PIN, DEBOUNCE_MS);
SensorToButton button1(BUTTON_PIN1, DEBOUNCE_MS);
SensorToButton button2(BUTTON_PIN2, DEBOUNCE_MS);
SensorToButton button3(BUTTON_PIN3, DEBOUNCE_MS);
SensorToButton button4(BUTTON_PIN4, DEBOUNCE_MS);
SensorToButton button5(BUTTON_PIN5, DEBOUNCE_MS);
SensorToButton button6(BUTTON_PIN6, DEBOUNCE_MS);
SensorToButton button7(BUTTON_PIN7, DEBOUNCE_MS);
SensorToButton button8(BUTTON_PIN8, DEBOUNCE_MS);
SensorToButton button9(BUTTON_PIN9, DEBOUNCE_MS);
SensorToButton button10(BUTTON_PIN10, DEBOUNCE_MS);
SensorToButton button11(BUTTON_PIN11, DEBOUNCE_MS);
SensorToButton button12(BUTTON_PIN12, DEBOUNCE_MS);
SensorToButton button13(BUTTON_PIN13, DEBOUNCE_MS);
SensorToButton button14(BUTTON_PIN14, DEBOUNCE_MS);
SensorToButton button15(BUTTON_PIN15, DEBOUNCE_MS);
SensorToButton button16(BUTTON_PIN16, DEBOUNCE_MS);
SensorToButton button17(BUTTON_PIN17, DEBOUNCE_MS);

//Declare counter for wave type
int Wave = 0;

void setup() {
  // put your setup code here, to run once:

  //Initialize the starting state of the piano keys
  pianoKey.begin(CHB, Wave, ENVELOPE0, 0);
  pianoKey2.begin(Wave, ENVELOPE0, 0);
  pianoKey3.begin(Wave, ENVELOPE0, 0);
  pianoKey4.begin(Wave, ENVELOPE0, 0);
  pianoKey5.begin(Wave, ENVELOPE0, 0);
  pianoKey6.begin(Wave, ENVELOPE0, 0);
  pianoKey7.begin(Wave, ENVELOPE0, 0);
  pianoKey8.begin(Wave, ENVELOPE0, 0);
  pianoKey9.begin(Wave, ENVELOPE0, 0);
  pianoKey10.begin(Wave, ENVELOPE0, 0);
  pianoKey11.begin(Wave, ENVELOPE0, 0);
  pianoKey12.begin(Wave, ENVELOPE0, 0);
  pianoKey13.begin(Wave, ENVELOPE0, 0);
  pianoKey14.begin(Wave, ENVELOPE0, 0);
  pianoKey15.begin(Wave, ENVELOPE0, 0);
  //pianoKey16.begin(TRIANGLE, ENVELOPE0, 0);
  //pianoKey17.begin(TRIANGLE, ENVELOPE0, 0);

  //Pause all piano keys so that they do not immediately play on setup
  pianoKey.pause(true);
  pianoKey2.pause(true); 
  pianoKey3.pause(true); 
  pianoKey4.pause(true); 
  pianoKey5.pause(true); 
  pianoKey6.pause(true);
  pianoKey7.pause(true);
  pianoKey8.pause(true);
  pianoKey9.pause(true);
  pianoKey10.pause(true);
  pianoKey11.pause(true);
  pianoKey12.pause(true);
  pianoKey13.pause(true);
  pianoKey14.pause(true);
  pianoKey15.pause(true);
  //pianoKey16.pause(true);
  //pianoKey17.pause(true);
}

void loop() {
  // put your main code here, to run repeatedly:
  //Update all the piano keys to the current state
  pianoKey.update();
  pianoKey2.update();
  pianoKey3.update();
  pianoKey4.update();
  pianoKey5.update();
  pianoKey6.update();
  pianoKey7.update();
  pianoKey8.update();
  pianoKey9.update();
  pianoKey10.update();
  pianoKey11.update();
  pianoKey12.update();
  pianoKey13.update();
  pianoKey14.update();
  pianoKey15.update();
  //pianoKey16.update();
  //pianoKey17.update();

  //get button status from all buttons
  toneSelect.read();
  button1.read();
  button2.read();
  button3.read();
  button4.read();
  button5.read();
  button6.read();
  button7.read();
  button8.read();
  button9.read();
  button10.read();
  button11.read();
  button12.read();
  button13.read();
  button14.read();
  button15.read();
  //button16.read();
  //button17.read();
  
  //executes the "play" function if button is pressed
  buttonToPiano(button1, pianoKey);
  buttonToPiano(button2, pianoKey2);
  buttonToPiano(button3, pianoKey3);
  buttonToPiano(button4, pianoKey4);
  buttonToPiano(button5, pianoKey5);
  buttonToPiano(button6, pianoKey6);
  buttonToPiano(button7, pianoKey7);
  buttonToPiano(button8, pianoKey8);
  buttonToPiano(button9, pianoKey9);
  buttonToPiano(button10, pianoKey10);
  buttonToPiano(button11, pianoKey11);
  buttonToPiano(button12, pianoKey12);
  buttonToPiano(button13, pianoKey13);
  buttonToPiano(button14, pianoKey14);
  buttonToPiano(button15, pianoKey15);
  //buttonToPiano(button16, pianoKey16);
  //buttonToPiano(button17, pianoKey17);

  //Changes the wave type if tone select button is pressed
  Wave = buttonToToneSelect(toneSelect, Wave);
  //Sets all the piano keys to the current wave type
  pianoKey.setWave(Wave);
  pianoKey2.setWave(Wave);
  pianoKey3.setWave(Wave);
  pianoKey4.setWave(Wave);
  pianoKey5.setWave(Wave);
  pianoKey6.setWave(Wave);
  pianoKey7.setWave(Wave);
  pianoKey8.setWave(Wave);
  pianoKey9.setWave(Wave);
  pianoKey10.setWave(Wave);
  pianoKey11.setWave(Wave);
  pianoKey12.setWave(Wave);
  pianoKey13.setWave(Wave);
  pianoKey14.setWave(Wave);
  pianoKey15.setWave(Wave);
  //pianoKey16.setWave(Wave);
  //pianoKey17.setWave(Wave);

}

//If button is pressed, plays the associated piano key
void buttonToPiano(SensorToButton b, MusicWithoutDelay &m) {  //Must put & before object, because we must point to the address of the Object in the Arduino Memory
  if (b.isPressed()) {
    m.play(1);    //play one time
  }
}

//If tone select button is pressed, changes the wave type (options are 0-6, but I am skipping 3 and 6)
int buttonToToneSelect(SensorToButton b, int Wave) {
    if(b.wasPressed()) {
        Wave = Wave + 1;
        Serial.print(Wave);
    }
    if(Wave == 3) {
        Wave = Wave + 1;
    }
    if(Wave == 6) {
        Wave = 0;
    }
    return Wave;
}

